package ru.dbotthepony.mc.otm.android.feature;

import net.minecraft.nbt.CompoundTag;
import net.minecraft.server.level.ServerPlayer;
import net.minecraftforge.event.entity.living.LivingHurtEvent;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.android.AndroidFeature;
import ru.dbotthepony.mc.otm.android.AndroidFeatureType;
import ru.dbotthepony.mc.otm.capability.IAndroidCapability;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;

import java.math.BigDecimal;

public class AndroidNanobotsArmor extends AndroidFeature {
	public AndroidNanobotsArmor(AndroidFeatureType<?> type, IAndroidCapability capability) {
		super(type, capability);
	}

	public int getStrength() {
		return strength;
	}

	public void setStrength(int strength) {
		this.strength = Math.max(0, Math.min(2, strength));
	}

	public int getSpeed() {
		return speed;
	}

	public void setSpeed(int speed) {
		this.speed = Math.max(0, Math.min(2, speed));
	}

	protected int strength = 0;
	protected int speed = 0;

	protected int ticks_passed = 0;
	protected int layers = 0;

	protected static final BigDecimal ENERGY_PER_BUILT = new BigDecimal(200);
	protected static final BigDecimal ENERGY_PER_HITPOINT = new BigDecimal(500);

	public static final int[] TICKS = new int[] {
		80, // 4 seconds to build a layer
		70, // 3.5 seconds to build a layer
		60, // 3 seconds to build a layer
		50, // 2.5 seconds to build a layer
	};

	public static final float[] SHIELD_STRENGTH = new float[] {
		0.1f,
		0.2f,
		0.3f,
		0.4f,
	};

	@Override
	public void tickServer() {
		if (layers < strength + 1 && capability.extractEnergyInner(ENERGY_PER_BUILT, true).compareTo(ENERGY_PER_BUILT) == 0) {
			ticks_passed++;

			if (ticks_passed >= TICKS[speed]) {
				layers++;
				capability.extractEnergyInner(ENERGY_PER_BUILT, false);
			}
		} else {
			ticks_passed = 0;
		}
	}

	@Override
	public void onHurt(LivingHurtEvent event) {
		ticks_passed = 0;

		if (!event.getSource().isBypassArmor() && layers > 0) {
			var absorbed = event.getAmount() * SHIELD_STRENGTH[layers];

			if (absorbed > 0.1f) {
				var required = ENERGY_PER_HITPOINT.multiply(new BigDecimal(Float.toString(absorbed)));
				var extracted = capability.extractEnergyInner(required, false);
				var real_absorbed = absorbed * extracted.divide(required, MatteryCapability.ROUND_RULES).floatValue();
				event.setAmount(event.getAmount() - real_absorbed);

				if (capability.getEntity() instanceof ServerPlayer ply) {
					ply.awardStat(Registry.Names.DAMAGE_ABSORBED, Math.round(real_absorbed * 10f));
				}

				layers--;
			}
		}
	}

	@Override
	public void serializeNBT(CompoundTag tag) {
		super.serializeNBT(tag);
		tag.putInt("ticks_passed", ticks_passed);
		tag.putInt("layers", layers);
		tag.putInt("strength", strength);
		tag.putInt("speed", speed);
	}

	@Override
	public void deserializeNBT(CompoundTag tag) {
		super.deserializeNBT(tag);

		ticks_passed = tag.getInt("ticks_passed");
		layers = tag.getInt("layers");
		strength = tag.getInt("strength");
		speed = tag.getInt("speed");
	}
}
