package ru.dbotthepony.mc.otm.block;

import com.google.common.collect.ImmutableMap;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.world.item.context.BlockPlaceContext;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.EntityBlock;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.level.block.state.properties.EnumProperty;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.world.phys.shapes.Shapes;
import net.minecraft.world.phys.shapes.VoxelShape;
import ru.dbotthepony.mc.otm.block.entity.BlockEntityMatterPanel;

import javax.annotation.Nullable;

public class BlockMatterPanel extends BlockMattery implements EntityBlock {
	@Nullable
	@Override
	public BlockEntity newBlockEntity(BlockPos blockPos, BlockState blockState) {
		return new BlockEntityMatterPanel(blockPos, blockState);
	}

	public static final EnumProperty<Direction> FACING = EnumProperty.create(
		"facing",
		Direction.class);

	private ImmutableMap<BlockState, VoxelShape> SHAPES;

	public BlockMatterPanel() {
		super();

		registerDefaultState(this.getStateDefinition().any().setValue(FACING, Direction.SOUTH));

		SHAPES = getShapeForEachState(blockstate -> {
			return switch (blockstate.getValue(FACING)) {
				case NORTH -> Shapes.box(
					0.0d,
					0.0d,
					0.625d,

					1.0d,
					1.0d,
					1.0d
				);
				case EAST -> Shapes.box(
					0.0d,
					0.0d,
					0.0d,

					0.375d,
					1.0d,
					1.0d
				);
				case WEST -> Shapes.box(
					0.625d,
					0.0d,
					0.0d,

					1.0d,
					1.0d,
					1.0d
				);
				case DOWN -> Shapes.box(
					0.0d,
					0.625d,
					0.0d,

					1.0d,
					1.0d,
					1.0d
				);
				case UP -> Shapes.box(
					0.0d,
					0.0d,
					0.0d,

					1.0d,
					0.375d,
					1.0d
				);
				default -> Shapes.box(
					0.0d,
					0.0d,
					0.0d,

					1.0d,
					1.0d,
					0.375d
				);
			};
		});
	}

	@Override
	public VoxelShape getShape(BlockState p_60555_, BlockGetter p_60556_, BlockPos p_60557_, CollisionContext p_60558_) {
		return SHAPES.get(p_60555_);
	}

	@Override
	protected void createBlockStateDefinition(StateDefinition.Builder<Block, BlockState> builder) {
		builder.add(FACING);
	}

	@Nullable
	@Override
	public BlockState getStateForPlacement(BlockPlaceContext context) {
		return this.defaultBlockState().setValue(FACING, context.getClickedFace());
	}

}
