package ru.dbotthepony.mc.otm.menu;

import net.minecraft.core.BlockPos;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.inventory.*;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.block.entity.BlockEntity;
import ru.dbotthepony.mc.otm.menu.slot.MatterySlot;
import ru.dbotthepony.mc.otm.menu.widget.AbstractWidget;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.function.Consumer;

public abstract class MatteryMenu extends AbstractContainerMenu {
	protected BlockEntity tile;

	protected Inventory inventory;
	protected Player ply;

	public final ArrayList<AbstractWidget> mattery_widgets = new ArrayList<>();
	public final ArrayList<MatterySlot> inventory_slots = new ArrayList<>();
	public final ArrayList<MatterySlot> main_slots = new ArrayList<>();

	protected final Set<Integer> locked_inventory_slots = new HashSet<>();

	protected boolean isInventorySlotLocked(int index) {
		return locked_inventory_slots.contains(index);
	}

	@Nullable
	protected ContainerSynchronizer synchronizer;

	@Override
	public void setSynchronizer(ContainerSynchronizer p_150417_) {
		synchronizer = p_150417_;
		super.setSynchronizer(p_150417_);
	}

	protected MatteryMenu(@Nullable MenuType<?> p_38851_, int p_38852_, Inventory inventory) {
		super(p_38851_, p_38852_);
		this.inventory = inventory;
		this.ply = inventory.player;
	}

	protected MatteryMenu(@Nullable MenuType<?> p_38851_, int p_38852_, Inventory inventory, BlockEntity tile) {
		this(p_38851_, p_38852_, inventory);
		this.tile = tile;
	}

	public void addWidget(AbstractWidget widget, Consumer<Consumer<ContainerData>> consumer) {
		if (mattery_widgets.contains(widget))
			return;

		mattery_widgets.add(widget);
		consumer.accept(this::addDataSlots);
	}

	public MatterySlot addMainSlot(MatterySlot slot) {
		addSlot(slot);
		main_slots.add(slot);
		return slot;
	}

	protected void addInventorySlots() {
		addInventorySlots(97);
	}

	protected int inventory_slot_index_start;
	protected int inventory_slot_index_end;

	protected void addInventorySlots(int offset) {
		boolean first = true;

		for (int i = 0; i < 3; ++i) {
			for (int j = 0; j < 9; ++j) {
				var slot = new MatterySlot(inventory, j + i * 9 + 9, 8 + j * 18, 14 + i * 18) {
					@Override
					public boolean mayPlace(ItemStack p_40231_) {
						return !isInventorySlotLocked(index);
					}

					@Override
					public boolean mayPickup(Player p_40228_) {
						return !isInventorySlotLocked(index);
					}
				};

				inventory_slots.add(slot);
				this.addSlot(slot);

				if (first) {
					first = false;
					inventory_slot_index_start = slot.index;
				}
			}
		}

		MatterySlot last = null;

		for (int k = 0; k < 9; ++k) {
			last = new MatterySlot(inventory, k, 8 + k * 18, 14 + 58) {
				@Override
				public boolean mayPlace(ItemStack p_40231_) {
					return !isInventorySlotLocked(index);
				}

				@Override
				public boolean mayPickup(Player p_40228_) {
					return !isInventorySlotLocked(index);
				}
			};

			this.addSlot(last);
			inventory_slots.add(last);
		}

		inventory_slot_index_end = last.index;
	}

	@Override
	public void broadcastChanges() {
		for (AbstractWidget widget : this.mattery_widgets) {
			widget.updateServer();
		}

		super.broadcastChanges();
	}

	@Override
	public void broadcastFullState() {
		for (AbstractWidget widget : this.mattery_widgets) {
			widget.updateServer();
		}

		super.broadcastFullState();
	}

	@Override
	public boolean stillValid(Player player) {
		if (tile == null)
			return true;

		if (player.level.getBlockEntity(tile.getBlockPos()) != tile) {
			return false;
		}

		BlockPos pos = tile.getBlockPos();
		return player.distanceToSqr((double)pos.getX() + 0.5D, (double)pos.getY() + 0.5D, (double)pos.getZ() + 0.5D) <= 64.0D;
	}

	abstract protected int getWorkingSlotStart();
	abstract protected int getWorkingSlotEnd();

	protected void notifySlotsWatchCurrentStack() {
		for (var slot : slots) {
			if (slot instanceof MatterySlot slot1) {
				slot1.watchCurrentStack();
			}
		}
	}

	protected void notifySlotsUnwatchCurrentStack() {
		for (var slot : slots) {
			if (slot instanceof MatterySlot slot1) {
				slot1.unwatchCurrentStack();
			}
		}
	}

	// This method receive Player interactor and slot_index where Shift + Right click occurred
	// It shall return item stack that got moved
	@Override
	public ItemStack quickMoveStack(Player ply, int slot_index) {
		// this.moveItemStackTo(ItemStack, int start_slot_index, int end_slot_index, boolean iteration_order)
		// returns boolean, telling whenever ItemStack was modified (moved or sharnk)
		// false means nothing happened
		// Last boolean determine order of slot iteration, where:
		// if TRUE, iteration order is end_slot_index -> start_slot_index
		// if FALSE iteration order is start_slot_index -> end_slot_index

		int start = getWorkingSlotStart();
		int end = getWorkingSlotEnd();

		if (start == end) {
			return ItemStack.EMPTY;
		}

		ItemStack moved = ItemStack.EMPTY;
		Slot get_slot = this.slots.get(slot_index);
		notifySlotsWatchCurrentStack();

		if (get_slot.hasItem()) {
			ItemStack slot_item = get_slot.getItem();
			moved = slot_item.copy();

			if (slot_index < inventory_slot_index_start) {
				// Moving FROM machine TO inventory

				if (!moveItemStackTo(slot_item, inventory_slot_index_start, inventory_slot_index_end + 1, false)) {
					notifySlotsUnwatchCurrentStack();
					return ItemStack.EMPTY;
				}
			} else if (!moveItemStackTo(slot_item, start, end, false)) {
				// Moving FROM inventory TO machine
				notifySlotsUnwatchCurrentStack();
				return ItemStack.EMPTY;
			}

			if (slot_item.isEmpty()) {
				get_slot.set(ItemStack.EMPTY);
			} else {
				get_slot.setChanged();
			}

			notifySlotsWatchCurrentStack();
		}

		notifySlotsUnwatchCurrentStack();
		return moved;
	}

	public MoveResult quickMoveToInventory(ItemStack stack, boolean simulate) {
		if (inventory_slot_index_start == 0 && inventory_slot_index_end == 0) {
			return new MoveResult(false, stack, Set.of());
		}

		return customMoveItemStackTo(stack, inventory_slot_index_start, inventory_slot_index_end + 1, false, simulate);
	}

	@Override
	protected boolean moveItemStackTo(ItemStack stack_to_move, int initial_slot, int final_slot, boolean reverse_direction) {
		var move_result = customMoveItemStackTo(stack_to_move, initial_slot, final_slot, reverse_direction, false);
		var remaining = move_result.remaining;

		if (remaining.getCount() == stack_to_move.getCount())
			return false;

		stack_to_move.setCount(remaining.getCount());
		return move_result.merge_occured;
	}

	public record MoveResult(boolean merge_occured, ItemStack remaining, Set<Slot> changed_slots) {

	}

	@Nonnull
	protected MoveResult customMoveItemStackTo(@Nonnull ItemStack stack_to_move, int initial_slot, int final_slot, boolean reverse_direction, boolean simulate) {
		boolean merge_occured = false;

		int i = reverse_direction ? final_slot - 1 : initial_slot;
		var changed = new HashSet<Slot>();

		stack_to_move = stack_to_move.copy();

		if (stack_to_move.isStackable()) {
			while (!stack_to_move.isEmpty()) {
				if (reverse_direction) {
					if (i < initial_slot) {
						break;
					}
				} else if (i >= final_slot) {
					break;
				}

				var slot = this.slots.get(i);
				var slot_stack = slot.getItem();

				if (!slot_stack.isEmpty() && ItemStack.isSameItemSameTags(stack_to_move, slot_stack)) {
					int j = slot_stack.getCount() + stack_to_move.getCount();
					int maxSize = slot.getMaxStackSize(stack_to_move);

					if (j <= maxSize) {
						stack_to_move.setCount(0);

						if (!simulate) {
							slot_stack.setCount(j);
							slot.setChanged();
						}

						merge_occured = true;
						changed.add(slot);
					} else if (slot_stack.getCount() < maxSize) {
						stack_to_move.shrink(maxSize - slot_stack.getCount());

						if (!simulate) {
							slot_stack.setCount(maxSize);
							slot.setChanged();
						}

						merge_occured = true;
						changed.add(slot);
					}
				}

				i += reverse_direction ? -1 : 1;
			}
		}

		if (!stack_to_move.isEmpty()) {
			i = reverse_direction ? final_slot - 1 : initial_slot;

			while (true) {
				if (reverse_direction) {
					if (i < initial_slot) {
						break;
					}
				} else if (i >= final_slot) {
					break;
				}

				var slot = this.slots.get(i);
				var slot_stack = slot.getItem();

				if (slot_stack.isEmpty() && slot.mayPlace(stack_to_move)) {
					if (!simulate) {
						if (stack_to_move.getCount() > slot.getMaxStackSize()) {
							slot.set(stack_to_move.split(slot.getMaxStackSize()));
						} else {
							slot.set(stack_to_move.split(stack_to_move.getCount()));
						}

						slot.setChanged();
					}

					merge_occured = true;
					changed.add(slot);
					break;
				}

				i += reverse_direction ? -1 : 1;
			}
		}

		return new MoveResult(merge_occured, stack_to_move, changed);
	}
}
