package ru.dbotthepony.mc.otm.network.android;

import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.server.level.ServerPlayer;
import net.minecraftforge.fmllegacy.network.NetworkEvent;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.android.AndroidResearch;
import ru.dbotthepony.mc.otm.android.AndroidResearchType;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.capability.AndroidCapabilityPlayer;
import ru.dbotthepony.mc.otm.menu.AndroidStationMenu;

import java.util.Objects;
import java.util.function.Supplier;

public record AndroidResearchRequestPacket(AndroidResearchType<?> research) {
	public AndroidResearchRequestPacket(AndroidResearchType<?> research) {
		this.research = research;
	}

	public AndroidResearchRequestPacket(AndroidResearch feature) {
		this(feature.type);
	}

	public void write(FriendlyByteBuf buffer) {
		buffer.writeInt(Registry.ANDROID_RESEARCH.getID(research));
	}

	public void play(Supplier<NetworkEvent.Context> context) {
		context.get().setPacketHandled(true);
		context.get().enqueueWork(() -> context.get().getSender().getCapability(MatteryCapability.ANDROID).ifPresent(cap -> this.playServer((AndroidCapabilityPlayer) cap, context.get().getSender())));
	}

	public void playServer(AndroidCapabilityPlayer cap, ServerPlayer ply) {
		if (!cap.isAndroid())
			return;

		if (ply.containerMenu instanceof AndroidStationMenu)
			cap.getResearch(research).research();
	}

	public static AndroidResearchRequestPacket read(FriendlyByteBuf buffer) {
		return new AndroidResearchRequestPacket(Objects.requireNonNull(Registry.ANDROID_RESEARCH.getValue(buffer.readInt())));
	}
}
